﻿// sets the initial values for the add or update record dialog.
function InitializeAddUpdateRecordDialog() {
    $('#addUpdateRecordDialog').dialog({
        autoOpen: false,
        resizable: false,
        width: 900,
        height: 500,
        modal: true
    });
}

// closes the '#addUpdateRecordDialog dialog.
function closeDialog() {
    $('#addUpdateRecordDialog').dialog('close');
}

// saves the serialized item to the database by calling a method in the controller.
function saveItem() {
    if (isDataValid()) {
        var serializedData = getSerializedData();
        callWebMethod(urlAndMethod + '&serializedData={' + serializedData + '}', null);
    }
}

// shows a delete dialog, clicking the Delete button will delete the item.
function deleteItem(itemID, url) {
    var ids = itemID;

    if (itemID.includes('?') && itemID.includes('?'))
        ids = itemID.replace('?', '').replace('&', ' and ');

    var dialogTitle = 'Permanently Delete Item ' + ids + '?';

    $("#errorConfirmationDialog").html('<p><span class="ui-icon ui-icon-alert" style="float:left; margin:0 7px 20px 0;"></span>Please click delete to confirm deletion of the following item:<br><br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' + ids + '</p>');

    $("#errorConfirmationDialog").dialog({
        title: dialogTitle,
        modal: true,
        buttons: {
            "Delete": function () {
                callWebMethod(url + itemID, '');
                $(this).dialog("close");
            },
            "Cancel": function () { $(this).dialog("close"); }
        }
    });

    $('#errorConfirmationDialog').dialog('open');
    return false;
}

// shows or hides items for the Add or Update operation.
function showHideItem(itemID) {
    if (itemID === null) {
        // add
        $('#addUpdateRecordDialog').dialog('open');
        $("#addUpdateRecordDialog").dialog("option", "title", "Add New " + addEditTitle);
        $("#inputAdd").show();                      
        $("#inputUpdate").hide();                   

        if ($("#trPrimaryKey") !== null)
            $("#trPrimaryKey").hide();
    }
    else {
        // update
        $('#addUpdateRecordDialog').dialog('open');
        $("#addUpdateRecordDialog").dialog("option", "title", "Edit " + addEditTitle + " " + itemID);
        $("#inputUpdate").show();                     
        $("#inputAdd").hide();                        

        if ($("#trPrimaryKey") !== null)
            $("#trPrimaryKey").show();
    }
}

// resets validation errors to their initial values
function resetValidationErrors() {
    $('.field-validation-error').removeClass('field-validation-error').addClass('field-validation-valid');
    $('.input-validation-error').removeClass('input-validation-error').addClass('input-validation-valid');
}

// calls a method in the controller (c# code).
// when the operation is successfull, it reloads the jqgrid's data
// and closes an open dialog.
function callWebMethod(url, parameter) {
    jQuery.ajax({
        type: "POST",
        url: url,
        data: "{" + parameter + "}",
        contentType: "application/json; charset=utf-8",
        dataType: "json",
        success: function (msg) {
            $("#list-grid").trigger("reloadGrid");
            closeDialog();
        },
        error: function (xhr, textStatus, errorThrown) {
            ShowError(xhr.responseText, '');
        }
    });
}

// shows an error message in a dialog.
function ShowError(errorMessage, errorTitle) {
    $(document).ready(function () {
        $("#errorDialog").html(errorMessage);
        $("#errorDialog").dialog({
            modal: true,
            title: errorTitle,
            buttons: {
                Ok: function () {
                    $(this).dialog("close");
                }
            }
        });
    });
}

// validates whether the passed value is numeric
// e.g. 1234 = true, 12abc34 = false.
function isNumeric(value) {
    var numericPattern = /^\d+$/;
    var matchArray = value.toString().match(numericPattern);

    if (matchArray !== null)
        return true;

    return false;
}

// validates whether the passed value is a decimal
// e.g. 12.34 = true, 12ab.34 = false.
function isDecimal(value) {
    var numericPattern = /^[0-9]+(\.[0-9]+)?$/;
    var matchArray = value.toString().match(numericPattern);

    if (matchArray !== null)
        return true;

    return false;
}

// validates whether the passed value is a date in any of these formats: 
// mm/dd/yyyy, m/d/yyyy, m/dd/yyyy, mm/d/yyyy
// e.g. 12/01/2018 = true, 12/1/2018 or 2018/12/01 = false.
function isDate(txtDate) {
    // checks for , 
    if (! /^\d\d\/\d\d\/\d\d\d\d$/.test(txtDate) && ! /^\d\/\d\/\d\d\d\d$/.test(txtDate) && ! /^\d\/\d\d\/\d\d\d\d$/.test(txtDate) && ! /^\d\d\/\d\/\d\d\d\d$/.test(txtDate))
        return false;

    const parts = txtDate.split('/').map((p) => parseInt(p, 10));
    parts[0] -= 1;
    const d = new Date(parts[2], parts[0], parts[1]);
    return d.getMonth() === parts[0] && d.getDate() === parts[1] && d.getFullYear() === parts[2];
}

// validates whether the passed value is within the range of -32768 and 32767.
function isWithinInt16Range(number) {
    if (number >= -32768 && number <= 32767)
        return true;

    return false;
}

// validates whether the passed value is within the range of -2147483648 and 2147483647.
function isWithinInt32Range(number) {
    if (number >= -2147483648 && number <= 2147483647)
        return true;

    return false;
}

// validates whether the passed value is within the safe number integer number javascript can process.
function isWithinInt64Range(number) {
    if (Number.isSafeInteger(parseInt(number)))
        return true;

    return false;
}

// validates whether the passed value is within the range of -9223372036854775808 and 922337036854775807.
function isWithinMoneyRange(number) {
    if (number >= -9223372036854775808 && number <= 922337036854775807)
        return true;

    return false;
}

// validates whether the passed value is within the range of -214748.3648 and 214748.3647.
function isWithinSmallMoneyRange(number) {
    if (number >= -214748.3648 && number <= 214748.3647)
        return true;

    return false;
}