﻿// true = add operation, false = update operation.
var isAddingRow = true;

// some random string (a hack) used as an ID to determine
// whether to toggle the #editLink, #cancelLink, and #saveLink.
var tempId = '-100abc';

// used as a suffix when a new row is being added/created
// when the suffix of current Id == 'jqg', then the current operation is add (otherwise it's update).
// e.g. jqg1
var newRowSuffix = 'jqg';

// adds a new row with empty controls in the jqrid.  used in the add operation.
// disables/hides links for the add operation.
function addRow() {
    if (isAddingRow) {
        $('#list-grid').addRow();
        $('#addLink1').addClass('disablelink');
        $('#addLink2').addClass('disablelink');
        toggleLinks(tempId);
        isAddingRow = false;
    }
}

// edits an existing row.
function editRow(currentId) {
    $('#list-grid').editRow(currentId);
    toggleLinks(currentId);
}

// cancels the add or update operation.
// restores the current row being edited, or restores the row being added.
function cancelRow(currentId) {
    $('#list-grid').restoreRow(currentId);
    toggleLinks(currentId);

    // cancel saving record
    if (currentId.indexOf(newRowSuffix) !== -1) {
        $('#addLink1').removeClass('disablelink');
        $('#addLink2').removeClass('disablelink');
        isAddingRow = true;
    }
}

// saves the row being added or edited.
function saveRow(currentId, controller, action) {
    var isValid = isDataValid(currentId);

    if (!isValid) {
        ShowError(errorMessage, 'An error occured when saving!');
        errorMessage = '';
        return false;
    }

    var serializedData = getSerializedData(currentId);

    // save record
    if (currentId.indexOf(newRowSuffix) !== -1) {
        callWebMethod('/' + controller + '/' + action + 'Add/', serializedData);

        $('#addLink1').removeClass('disablelink');
        $('#addLink2').removeClass('disablelink');
        isAddingRow = true;
    }
    else {
        // update               
        callWebMethod('/' + controller + '/' + action + 'Update/', serializedData);
    }

    $('#list-grid').saveRow(currentId);
    toggleLinks(currentId);
}

// toggles the #editLink, #cancelLink, #saveLink, and #deleteLink
// when an add or update operation is done.
function toggleLinks(currentId) {
    var ids = jQuery('#list-grid').jqGrid('getDataIDs');

    if (currentId === tempId)
        currentId = ids[0];

    $('#editLink' + currentId).toggle();
    $('#cancelLink' + currentId).toggle();
    $('#saveLink' + currentId).toggle();

    for (var i = 0; i < ids.length; i++) {
        if (ids[i] !== currentId)
            $('#editLink' + ids[i]).toggle();

        $('#deleteLink' + ids[i]).toggle();
    }
}

// shows a delete dialog, clicking the Delete button will delete the item.
function deleteItem(itemID) {
    var ids = itemID;

    if (itemID.includes('?') && itemID.includes('?'))
        ids = itemID.replace('?', '').replace('&', ' and ');

    var dialogTitle = 'Permanently Delete Item ' + ids + '?';

    $("#errorConfirmationDialog").html('<p><span class="ui-icon ui-icon-alert" style="float:left; margin:0 7px 20px 0;"></span>Please click delete to confirm deletion of the following item:<br><br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' + ids + '</p>');
   
    $("#errorConfirmationDialog").dialog({
        title: dialogTitle,
        modal: true,
        buttons: {
            "Delete": function () {
                callWebMethod(urlAndMethod + itemID, '');
                $(this).dialog("close");
            },
            "Cancel": function () { $(this).dialog("close"); }
        }
    });

    $('#errorConfirmationDialog').dialog('open');
    return false;
}

// calls a method in the controller (c# code).
// when the operation is successfull, it reloads the jqgrid's data.
function callWebMethod(urlAndMethod, parameter) {
    jQuery.ajax({
        type: "POST",
        url: urlAndMethod,
        data: JSON.stringify(parameter),
        contentType: "application/json; charset=utf-8",
        dataType: "json",
        success: function (msg) {
            $("#list-grid").trigger("reloadGrid");
        },
        error: function (xhr, textStatus, errorThrown) {
            ShowError(xhr.responseText, '');
        }
    });
}

// shows an error message in a dialog.
function ShowError(errorMessage, errorTitle) {
    $(document).ready(function () {
        $("#errorDialog").html(errorMessage);
        $("#errorDialog").dialog({
            modal: true,
            title: errorTitle,
            buttons: {
                Ok: function () {
                    $(this).dialog("close");
                }
            }
        });
    });
}

// validates whether the passed value is numeric
// e.g. 1234 = true, 12abc34 = false.
function isNumeric(value) {
    var numericPattern = /^\d+$/;
    var matchArray = value.toString().match(numericPattern);

    if (matchArray !== null)
        return true;

    return false;
}

// validates whether the passed value is a decimal
// e.g. 12.34 = true, 12ab.34 = false.
function isDecimal(value) {
    var numericPattern = /^[0-9]+(\.[0-9]+)?$/;
    var matchArray = value.toString().match(numericPattern);

    if (matchArray !== null)
        return true;

    return false;
}

// validates whether the passed value is a date in any of these formats: 
// mm/dd/yyyy, m/d/yyyy, m/dd/yyyy, mm/d/yyyy
// e.g. 12/01/2018 = true, 12/1/2018 or 2018/12/01 = false.
function isDate(txtDate) {
    // checks for , 
    if (! /^\d\d\/\d\d\/\d\d\d\d$/.test(txtDate) && ! /^\d\/\d\/\d\d\d\d$/.test(txtDate) && ! /^\d\/\d\d\/\d\d\d\d$/.test(txtDate) && ! /^\d\d\/\d\/\d\d\d\d$/.test(txtDate)) 
        return false;

    const parts = txtDate.split('/').map((p) => parseInt(p, 10));
    parts[0] -= 1;
    const d = new Date(parts[2], parts[0], parts[1]);
    return d.getMonth() === parts[0] && d.getDate() === parts[1] && d.getFullYear() === parts[2];
}

// get's today's date.
function getSelectedDate() {
    var currentTime = new Date();

    var month = parseInt(currentTime.getMonth() + 1);
    month = month <= 9 ? "0" + month : month;

    var day = currentTime.getDate();
    day = day <= 9 ? "0" + day : day;

    var year = currentTime.getFullYear();

    return day + '/' + month + "/" + year;
}