using System;
using StoredProcWaApi.Models;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace StoredProcWaApi.BusinessLayer
{
     /// <summary>
     /// Works like the Base interface for ICategoryBusinessLayer interface.
     /// **************************** Do not make changes to this interface ****************************
     /// ** Put your additional code in the ICategoryBusinessLayer interface under the BusinessLayer folder. **
     /// *******************************************************************************************
     /// </summary>
     public partial interface ICategoryBusinessLayer
     {

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         public Task<Category> SelectByPrimaryKeyAsync(int categoryID);

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         public Task<int> GetRecordCountAsync();

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         public Task<int> GetRecordCountDynamicWhereAsync(int? categoryID, string categoryName);

         /// <summary>
         /// Selects records as a collection (List) of Category sorted by the sortByExpression.
         /// </summary>
         public Task<List<Category>> SelectSkipAndTakeAsync(int rows, int startRowIndex, string sortByExpression);

         /// <summary>
         /// Selects records as a collection (List) of Category sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         public Task<List<Category>> SelectSkipAndTakeDynamicWhereAsync(int? categoryID, string categoryName, int rows, int startRowIndex, string sortByExpression);

         /// <summary>
         /// Selects all records as a collection (List) of Category
         /// </summary>
         public Task<List<Category>> SelectAllAsync();

         /// <summary>
         /// Selects all records as a collection (List) of Category sorted by the sort expression
         /// </summary>
         public Task<List<Category>> SelectAllAsync(string sortByExpression);

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Category.
         /// </summary>
         public Task<List<Category>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName);

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Category sorted by the sort expression.
         /// </summary>
         public Task<List<Category>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName, string sortByExpression);

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         public Task<List<Category>> SelectCategoryDropDownListDataAsync();

         /// <summary>
         /// Sorts the List<Category >by sort expression
         /// </summary>
         public Task<List<Category>> SortByExpressionAsync(List<Category> objCategoriesList, string sortExpression);

         /// <summary>
         /// Inserts a new record
         /// </summary>
         public Task<int> InsertAsync(Category objCategory);

         /// <summary>
         /// Updates a record
         /// </summary>
         public Task UpdateAsync(Category objCategory);

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         public Task DeleteAsync(int categoryID);

         /// <summary>
         /// Deletes multiple records based on primary keys
         /// </summary>
         public Task DeleteMultipleAsync(List<Int32> categoryIDList);
     }
}
