using StoredProcWaApi.DataRepository.Helper;
using StoredProcWaApi.Models;
using System;
using System.Data;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace StoredProcWaApi.DataRepository
{
     /// <summary>
     /// Works like the Base class for CategoryRepository class.
     /// ************************************* Do not make changes to this class *************************************
     /// ** Put your additional code in the CategoryRepository class under the DataRepository folder. **
     /// *************************************************************************************************************
     /// </summary>
     public partial class CategoryRepository : ICategoryRepository
     {
         private const CommandType _commandType = CommandType.StoredProcedure;
         private readonly string _connectionString;

         internal CategoryRepository(string connectionString)
         {
             _connectionString = connectionString;
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         async Task<DataTable> ICategoryRepository.SelectByPrimaryKeyAsync(int categoryID)
         {
             string storedProcName = "[dbo].[acg6mvc_Categories_SelectByPrimaryKey]";
             List<SqlParameter> sqlParamList = new();

             // add parameters to the sqlParams
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@categoryID", categoryID);

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         async Task<int> ICategoryRepository.GetRecordCountAsync()
         {
              string storedProcName = "[dbo].[acg6mvc_Categories_GetRecordCount]";
              return await this.GetRecordCountSharedAsync(storedProcName, null, null);
         }

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         async Task<int> ICategoryRepository.GetRecordCountDynamicWhereAsync(int? categoryID, string categoryName)
         {
             List<SqlParameter> sqlParamList = new();
             string storedProcName = "[dbo].[acg6mvc_Categories_GetRecordCountWhereDynamic]";
             int recordCount = 0;

             // search parameters
             this.AddSearchCommandParamsShared(sqlParamList, categoryID, categoryName);

             // get the data
             DataTable dt = await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);

             // convert data to an int - record count
             if (dt is not null && dt.Rows.Count > 0)
                 recordCount = (int)dt.Rows[0]["RecordCount"];

             return recordCount;
         }

         /// <summary>
         /// Selects Categories table records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of rows)
         /// </summary>
         async Task<DataTable> ICategoryRepository.SelectSkipAndTakeAsync(string sortByExpression, int startRowIndex, int rows)
         {
             string storedProcName = "[dbo].[acg6mvc_Categories_SelectSkipAndTake]";
             return await this.SelectSharedAsync(storedProcName, null, null, sortByExpression, startRowIndex, rows);
         }

         /// <summary>
         /// Selects Categories table records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records) based on search parameters
         /// </summary>
         async Task<DataTable> ICategoryRepository.SelectSkipAndTakeDynamicWhereAsync(int? categoryID, string categoryName, string sortByExpression, int startRowIndex, int rows)
         {
             List<SqlParameter> sqlParamList = new();
             string storedProcName = "[dbo].[acg6mvc_Categories_SelectSkipAndTakeWhereDynamic]";

             // select, skip, take, sort parameters
             DatabaseFunctions.AddSelectSkipAndTakeParams(sqlParamList, sortByExpression, startRowIndex, rows);

             // search parameters
             this.AddSearchCommandParamsShared(sqlParamList, categoryID, categoryName);

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString,  storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Selects all Categories
         /// </summary>
         async Task<DataTable> ICategoryRepository.SelectAllAsync()
         {
             string storedProcName = "[dbo].[acg6mvc_Categories_SelectAll]";
             return await this.SelectSharedAsync(storedProcName, String.Empty, null, null, null, null);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Category.
         /// </summary>
         async Task<DataTable> ICategoryRepository.SelectAllDynamicWhereAsync(int? categoryID, string categoryName)
         {
             string storedProcName = "[dbo].[acg6mvc_Categories_SelectAllWhereDynamic]";
             List<SqlParameter> sqlParamList = new();

             // search parameters
             this.AddSearchCommandParamsShared(sqlParamList, categoryID, categoryName);

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control
         /// </summary>
         async Task<DataTable> ICategoryRepository.SelectCategoryDropDownListDataAsync()
         {
             List<SqlParameter> sqlParamList = new();
             string storedProcName = "[dbo].[acg6mvc_Categories_SelectDropDownListData]";

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         async Task ICategoryRepository.DeleteAsync(int categoryID)
         {
             List<SqlParameter> sqlParamList = new();
             string storedProcName = "[dbo].[acg6mvc_Categories_Delete]";

             DatabaseFunctions.AddSqlParameter(sqlParamList, "@categoryID", categoryID);

             // delete record
             await DatabaseFunctions.ExecuteSqlCommandAsync(_connectionString, storedProcName, sqlParamList, _commandType, DatabaseOperationType.Delete);
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         async Task<int> ICategoryRepository.InsertAsync(Category objCategory)
         {
             return await this.InsertUpdateAsync(objCategory, DatabaseOperationType.Create);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         async Task ICategoryRepository.UpdateAsync(Category objCategory) =>
             await this.InsertUpdateAsync(objCategory, DatabaseOperationType.Update);

         /// <summary>
         /// Creates a new Category or Updates an existing Category base on the Operation Type.
         /// </summary>
         /// <param name="objCategory">The data to Create or Update</param>
         /// <param name="operationType">Accepts Create or Update only</param>
         /// <returns>Newly Created CategoryID for create, or the CategoryID for the record to be updated</returns>
         private async Task<int> InsertUpdateAsync(Category objCategory, DatabaseOperationType operationType)
         {
             if (operationType == DatabaseOperationType.RetrieveDataTable || operationType == DatabaseOperationType.Delete)
                 throw new ArgumentException("Invalid DatabaseOperationType!  Acceptable operation types are: Create or Update only.", "operationType: " + operationType.ToString());

             List<SqlParameter> sqlParamList = new();

             // set values for the Category to be created or updated
             int newlyCreatedCategoryID = objCategory.CategoryID;

             // get the SQL script for the create or update operation
             string storedProcName;

             if (operationType == DatabaseOperationType.Update)
                 storedProcName = "[dbo].[acg6mvc_Categories_Update]";
             else
                 storedProcName = "[dbo].[acg6mvc_Categories_Insert]";

             object description = objCategory.Description;

             if (String.IsNullOrEmpty(objCategory.Description))
                 description = System.DBNull.Value;

             // update parameters
             if (operationType == DatabaseOperationType.Update)
             {
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@categoryID", objCategory.CategoryID);
             }

             DatabaseFunctions.AddSqlParameter(sqlParamList, "@categoryName", objCategory.CategoryName);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@description", description);

             // create a new record or update an existing record
             if (operationType == DatabaseOperationType.Update)
                 await DatabaseFunctions.ExecuteSqlCommandAsync(_connectionString, storedProcName, sqlParamList, _commandType, DatabaseOperationType.Update);
             else
                 newlyCreatedCategoryID = (int)await DatabaseFunctions.ExecuteSqlCommandAsync(_connectionString, storedProcName, sqlParamList, _commandType, DatabaseOperationType.Create, false);

             return newlyCreatedCategoryID;
         }

         /// <summary>
         /// Gets the Number of Records from the Categories table based on the SQL script.
         /// </summary>
         /// <param name="storedProcName">Stored Procedure Name</param>
         /// <param name="parameterName">Parameter Name used to create the SqlParameter</param>
         /// <param name="parameterValue">Parameter Value used to create the SqlParamater</param>
         /// <returns>Record Count - int</returns>
         private async Task<int> GetRecordCountSharedAsync(string storedProcName, string parameterName, object parameterValue)
         {
             List<SqlParameter> sqlParamList = new();

             return await DatabaseFunctions.GetRecordCountAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Selects records from the database.  Used by other methods that selects data from the database.
         /// </summary>
         /// <param name="storedProcName">Stored Procedure Name</param>
         /// <param name="parameterName">Parameter Name used to create the SqlParameter</param>
         /// <param name="parameterValue">Parameter Value used to create the SqlParamater</param>>
         /// <param name="sortByExpression">Sort by Column Name.  E.g. "CategoryID" sorts by CategoryID in ascending order.  "CategoryID desc" sorts by CategoryID in descending order.</param>
         /// <param name="startRowIndex">Optional.  Index of the row to start taking data from.  E.g. 0 will start taking data from the 1st row, 9 will start taking data from the 10th row.</param>
         /// <param name="rows">Optional.  Number of rows to take.</param>
         /// <returns>List of Categories</returns>
         private async Task<DataTable> SelectSharedAsync(string storedProcName, string parameterName, object parameterValue, string sortByExpression, int? startRowIndex, int? rows)
         {
             List<SqlParameter> sqlParamList = new();

             // select, skip, take, sort sql parameters
             if (!String.IsNullOrEmpty(sortByExpression) && startRowIndex is not null && rows is not null)
                 DatabaseFunctions.AddSelectSkipAndTakeParams(sqlParamList, sortByExpression, startRowIndex.Value, rows.Value);

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Adds search parameters to the List of SqlParameter
         /// </summary>
         private void AddSearchCommandParamsShared(List<SqlParameter> sqlParamList, int? categoryID, string categoryName)
         {
              if(categoryID is not null)
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@categoryID", categoryID);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@categoryID", System.DBNull.Value);

              if(!String.IsNullOrEmpty(categoryName))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@categoryName", categoryName);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@categoryName", System.DBNull.Value);

         }
     }
}
