using StoredProcWaApi.Models;
using System;
using System.Data;
using System.Threading.Tasks;

namespace StoredProcWaApi.DataRepository
{
     /// <summary>
     /// Works like the Base interface for ISupplierRepository interface.
     /// **************************** Do not make changes to this interface ****************************
     /// ** Put your additional code in the ISupplierRepository interface under the DataRepository folder. **
     /// ***********************************************************************************************
     /// </summary>
     public partial interface ISupplierRepository
     {
         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         internal Task<DataTable> SelectByPrimaryKeyAsync(int supplierID);

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         internal Task<int> GetRecordCountAsync();

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         internal Task<int> GetRecordCountDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax);

         /// <summary>
         /// Selects Suppliers table records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of rows)
         /// </summary>
         internal Task<DataTable> SelectSkipAndTakeAsync(string sortByExpression, int startRowIndex, int rows);

         /// <summary>
         /// Selects Suppliers table records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records) based on search parameters
         /// </summary>
         internal Task<DataTable> SelectSkipAndTakeDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sortByExpression, int startRowIndex, int rows);

         /// <summary>
         /// Selects all Suppliers
         /// </summary>
         internal Task<DataTable> SelectAllAsync();

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Suppliers.
         /// </summary>
         internal Task<DataTable> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax);

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control
         /// </summary>
         internal Task<DataTable> SelectSupplierDropDownListDataAsync();

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         internal Task DeleteAsync(int supplierID);

         /// <summary>
         /// Inserts a record
         /// </summary>
         internal Task<int> InsertAsync(Supplier objSupplier);

         /// <summary>
         /// Updates a record
         /// </summary>
         internal Task UpdateAsync(Supplier objSupplier);
     }
}
