using StoredProcWaApi.DataRepository.Helper;
using StoredProcWaApi.Models;
using System;
using System.Data;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace StoredProcWaApi.DataRepository
{
     /// <summary>
     /// Works like the Base class for SupplierRepository class.
     /// ************************************* Do not make changes to this class *************************************
     /// ** Put your additional code in the SupplierRepository class under the DataRepository folder. **
     /// *************************************************************************************************************
     /// </summary>
     public partial class SupplierRepository : ISupplierRepository
     {
         private const CommandType _commandType = CommandType.StoredProcedure;
         private readonly string _connectionString;

         internal SupplierRepository(string connectionString)
         {
             _connectionString = connectionString;
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         async Task<DataTable> ISupplierRepository.SelectByPrimaryKeyAsync(int supplierID)
         {
             string storedProcName = "[dbo].[acg6mvc_Suppliers_SelectByPrimaryKey]";
             List<SqlParameter> sqlParamList = new();

             // add parameters to the sqlParams
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@supplierID", supplierID);

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         async Task<int> ISupplierRepository.GetRecordCountAsync()
         {
              string storedProcName = "[dbo].[acg6mvc_Suppliers_GetRecordCount]";
              return await this.GetRecordCountSharedAsync(storedProcName, null, null);
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         async Task<int> ISupplierRepository.GetRecordCountDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             List<SqlParameter> sqlParamList = new();
             string storedProcName = "[dbo].[acg6mvc_Suppliers_GetRecordCountWhereDynamic]";
             int recordCount = 0;

             // search parameters
             this.AddSearchCommandParamsShared(sqlParamList, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

             // get the data
             DataTable dt = await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);

             // convert data to an int - record count
             if (dt is not null && dt.Rows.Count > 0)
                 recordCount = (int)dt.Rows[0]["RecordCount"];

             return recordCount;
         }

         /// <summary>
         /// Selects Suppliers table records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of rows)
         /// </summary>
         async Task<DataTable> ISupplierRepository.SelectSkipAndTakeAsync(string sortByExpression, int startRowIndex, int rows)
         {
             string storedProcName = "[dbo].[acg6mvc_Suppliers_SelectSkipAndTake]";
             return await this.SelectSharedAsync(storedProcName, null, null, sortByExpression, startRowIndex, rows);
         }

         /// <summary>
         /// Selects Suppliers table records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records) based on search parameters
         /// </summary>
         async Task<DataTable> ISupplierRepository.SelectSkipAndTakeDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sortByExpression, int startRowIndex, int rows)
         {
             List<SqlParameter> sqlParamList = new();
             string storedProcName = "[dbo].[acg6mvc_Suppliers_SelectSkipAndTakeWhereDynamic]";

             // select, skip, take, sort parameters
             DatabaseFunctions.AddSelectSkipAndTakeParams(sqlParamList, sortByExpression, startRowIndex, rows);

             // search parameters
             this.AddSearchCommandParamsShared(sqlParamList, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString,  storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Selects all Suppliers
         /// </summary>
         async Task<DataTable> ISupplierRepository.SelectAllAsync()
         {
             string storedProcName = "[dbo].[acg6mvc_Suppliers_SelectAll]";
             return await this.SelectSharedAsync(storedProcName, String.Empty, null, null, null, null);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Supplier.
         /// </summary>
         async Task<DataTable> ISupplierRepository.SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             string storedProcName = "[dbo].[acg6mvc_Suppliers_SelectAllWhereDynamic]";
             List<SqlParameter> sqlParamList = new();

             // search parameters
             this.AddSearchCommandParamsShared(sqlParamList, supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control
         /// </summary>
         async Task<DataTable> ISupplierRepository.SelectSupplierDropDownListDataAsync()
         {
             List<SqlParameter> sqlParamList = new();
             string storedProcName = "[dbo].[acg6mvc_Suppliers_SelectDropDownListData]";

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         async Task ISupplierRepository.DeleteAsync(int supplierID)
         {
             List<SqlParameter> sqlParamList = new();
             string storedProcName = "[dbo].[acg6mvc_Suppliers_Delete]";

             DatabaseFunctions.AddSqlParameter(sqlParamList, "@supplierID", supplierID);

             // delete record
             await DatabaseFunctions.ExecuteSqlCommandAsync(_connectionString, storedProcName, sqlParamList, _commandType, DatabaseOperationType.Delete);
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         async Task<int> ISupplierRepository.InsertAsync(Supplier objSupplier)
         {
             return await this.InsertUpdateAsync(objSupplier, DatabaseOperationType.Create);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         async Task ISupplierRepository.UpdateAsync(Supplier objSupplier) =>
             await this.InsertUpdateAsync(objSupplier, DatabaseOperationType.Update);

         /// <summary>
         /// Creates a new Supplier or Updates an existing Supplier base on the Operation Type.
         /// </summary>
         /// <param name="objSupplier">The data to Create or Update</param>
         /// <param name="operationType">Accepts Create or Update only</param>
         /// <returns>Newly Created SupplierID for create, or the SupplierID for the record to be updated</returns>
         private async Task<int> InsertUpdateAsync(Supplier objSupplier, DatabaseOperationType operationType)
         {
             if (operationType == DatabaseOperationType.RetrieveDataTable || operationType == DatabaseOperationType.Delete)
                 throw new ArgumentException("Invalid DatabaseOperationType!  Acceptable operation types are: Create or Update only.", "operationType: " + operationType.ToString());

             List<SqlParameter> sqlParamList = new();

             // set values for the Supplier to be created or updated
             int newlyCreatedSupplierID = objSupplier.SupplierID;

             // get the SQL script for the create or update operation
             string storedProcName;

             if (operationType == DatabaseOperationType.Update)
                 storedProcName = "[dbo].[acg6mvc_Suppliers_Update]";
             else
                 storedProcName = "[dbo].[acg6mvc_Suppliers_Insert]";

             object contactName = objSupplier.ContactName;
             object contactTitle = objSupplier.ContactTitle;
             object address = objSupplier.Address;
             object city = objSupplier.City;
             object region1 = objSupplier.Region1;
             object postalCode = objSupplier.PostalCode;
             object country = objSupplier.Country;
             object phone = objSupplier.Phone;
             object fax = objSupplier.Fax;
             object homePage = objSupplier.HomePage;

             if (String.IsNullOrEmpty(objSupplier.ContactName))
                 contactName = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.ContactTitle))
                 contactTitle = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.Address))
                 address = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.City))
                 city = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.Region1))
                 region1 = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.PostalCode))
                 postalCode = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.Country))
                 country = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.Phone))
                 phone = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.Fax))
                 fax = System.DBNull.Value;

             if (String.IsNullOrEmpty(objSupplier.HomePage))
                 homePage = System.DBNull.Value;

             // update parameters
             if (operationType == DatabaseOperationType.Update)
             {
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@supplierID", objSupplier.SupplierID);
             }

             DatabaseFunctions.AddSqlParameter(sqlParamList, "@companyName", objSupplier.CompanyName);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@contactName", contactName);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@contactTitle", contactTitle);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@address", address);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@city", city);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@region1", region1);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@postalCode", postalCode);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@country", country);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@phone", phone);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@fax", fax);
             DatabaseFunctions.AddSqlParameter(sqlParamList, "@homePage", homePage);

             // create a new record or update an existing record
             if (operationType == DatabaseOperationType.Update)
                 await DatabaseFunctions.ExecuteSqlCommandAsync(_connectionString, storedProcName, sqlParamList, _commandType, DatabaseOperationType.Update);
             else
                 newlyCreatedSupplierID = (int)await DatabaseFunctions.ExecuteSqlCommandAsync(_connectionString, storedProcName, sqlParamList, _commandType, DatabaseOperationType.Create, false);

             return newlyCreatedSupplierID;
         }

         /// <summary>
         /// Gets the Number of Records from the Suppliers table based on the SQL script.
         /// </summary>
         /// <param name="storedProcName">Stored Procedure Name</param>
         /// <param name="parameterName">Parameter Name used to create the SqlParameter</param>
         /// <param name="parameterValue">Parameter Value used to create the SqlParamater</param>
         /// <returns>Record Count - int</returns>
         private async Task<int> GetRecordCountSharedAsync(string storedProcName, string parameterName, object parameterValue)
         {
             List<SqlParameter> sqlParamList = new();

             return await DatabaseFunctions.GetRecordCountAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Selects records from the database.  Used by other methods that selects data from the database.
         /// </summary>
         /// <param name="storedProcName">Stored Procedure Name</param>
         /// <param name="parameterName">Parameter Name used to create the SqlParameter</param>
         /// <param name="parameterValue">Parameter Value used to create the SqlParamater</param>>
         /// <param name="sortByExpression">Sort by Column Name.  E.g. "SupplierID" sorts by SupplierID in ascending order.  "SupplierID desc" sorts by SupplierID in descending order.</param>
         /// <param name="startRowIndex">Optional.  Index of the row to start taking data from.  E.g. 0 will start taking data from the 1st row, 9 will start taking data from the 10th row.</param>
         /// <param name="rows">Optional.  Number of rows to take.</param>
         /// <returns>List of Suppliers</returns>
         private async Task<DataTable> SelectSharedAsync(string storedProcName, string parameterName, object parameterValue, string sortByExpression, int? startRowIndex, int? rows)
         {
             List<SqlParameter> sqlParamList = new();

             // select, skip, take, sort sql parameters
             if (!String.IsNullOrEmpty(sortByExpression) && startRowIndex is not null && rows is not null)
                 DatabaseFunctions.AddSelectSkipAndTakeParams(sqlParamList, sortByExpression, startRowIndex.Value, rows.Value);

             // get and return the data
             return await DatabaseFunctions.GetDataTableAsync(_connectionString, storedProcName, sqlParamList, _commandType);
         }

         /// <summary>
         /// Adds search parameters to the List of SqlParameter
         /// </summary>
         private void AddSearchCommandParamsShared(List<SqlParameter> sqlParamList, int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
              if(supplierID is not null)
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@supplierID", supplierID);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@supplierID", System.DBNull.Value);

              if(!String.IsNullOrEmpty(companyName))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@companyName", companyName);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@companyName", System.DBNull.Value);

              if(!String.IsNullOrEmpty(contactName))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@contactName", contactName);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@contactName", System.DBNull.Value);

              if(!String.IsNullOrEmpty(contactTitle))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@contactTitle", contactTitle);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@contactTitle", System.DBNull.Value);

              if(!String.IsNullOrEmpty(address))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@address", address);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@address", System.DBNull.Value);

              if(!String.IsNullOrEmpty(city))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@city", city);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@city", System.DBNull.Value);

              if(!String.IsNullOrEmpty(region1))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@region1", region1);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@region1", System.DBNull.Value);

              if(!String.IsNullOrEmpty(postalCode))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@postalCode", postalCode);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@postalCode", System.DBNull.Value);

              if(!String.IsNullOrEmpty(country))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@country", country);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@country", System.DBNull.Value);

              if(!String.IsNullOrEmpty(phone))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@phone", phone);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@phone", System.DBNull.Value);

              if(!String.IsNullOrEmpty(fax))
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@fax", fax);
              else
                  DatabaseFunctions.AddSqlParameter(sqlParamList, "@fax", System.DBNull.Value);

         }
     }
}
